
from functools import wraps

from django import template


class Library(template.Library):
    """
    Extends django.template.Library providing several shortcuts that attempt 
    to take the leg-work out of creating different types of template tags.
    """

    def as_tag(self, tag_func):
        """
        Creates a tag expecting the format: ``{% tag_name as var_name %}``
        The decorated func returns the value that is given to ``var_name`` in
        the template.
        """
        @wraps(tag_func)
        def tag_wrapper(parser, token):
            class AsTagNode(template.Node):
                def render(self, context):
                    parts = token.split_contents()
                    context[parts[-1]] = tag_func(*parts[1:-2])
                    return ""
            return AsTagNode()
        return self.tag(tag_wrapper)

    def render_tag(self, tag_func):
        """
        Creates a tag using the decorated func as the render function for the
        template tag node. The render function takes two arguments - the
        template context and the tag token.
        """
        @wraps(tag_func)
        def tag_wrapper(parser, token):
            class RenderTagNode(template.Node):
                def render(self, context):
                    return tag_func(context, token)
            return RenderTagNode()
        return self.tag(tag_wrapper)

    def to_end_tag(self, tag_func):
        """
        Creates a tag that parses until it finds the corresponding end tag,
        eg: for a tag named ``mytag`` it will parse until ``endmytag``.
        The decorated func's return value is used to render the parsed
        content and takes three arguments - the parsed content between the
        start and end tags, the template context and the tag token.
        """
        @wraps(tag_func)
        def tag_wrapper(parser, token):
            class ToEndTagNode(template.Node):
                def __init__(self):
                    end_name = "end%s" % tag_func.__name__
                    self.nodelist = parser.parse((end_name,))
                    parser.delete_first_token()
                def render(self, context):
                    args = (self.nodelist.render(context), context, token)
                    return tag_func(*args[:tag_func.func_code.co_argcount])
            return ToEndTagNode()
        return self.tag(tag_wrapper)




from django.contrib.admin.views.decorators import staff_member_required
from django.db.models import get_model
from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _

from mezzanine.core.forms import get_edit_form
from mezzanine.core.models import Keyword, Displayable
from mezzanine.settings import SEARCH_MAX_PAGING_LINKS, SEARCH_PER_PAGE
from mezzanine.utils import is_editable, paginate


def admin_keywords_submit(request):
    """
    Adds any new given keywords from the custom keywords field in the admin and
    returns their IDs for use when saving a model with a keywords field.
    """
    ids = []
    for title in request.POST.get("text_keywords", "").split(","):
        title = "".join([c for c in title if c.isalnum() or c == "-"]).lower()
        if title:
            keyword, created = Keyword.objects.get_or_create(title=title)
            ids.append(str(keyword.id))
    return HttpResponse(",".join(set(ids)))
admin_keywords_submit = staff_member_required(admin_keywords_submit)


def search(request, template="search_results.html"):
    """
    Display search results.
    """
    query = request.GET.get("q", "")
    results = Displayable.objects.search(query)
    results = paginate(results, request.GET.get("page", 1), SEARCH_PER_PAGE,
        SEARCH_MAX_PAGING_LINKS)
    context = {"query": query, "results": results}
    return render_to_response(template, context, RequestContext(request))


def edit(request):
    model = get_model(request.POST["app"], request.POST["model"])
    obj = model.objects.get(id=request.POST["id"])
    form = get_edit_form(obj, request.POST["attr"], data=request.POST)
    if not is_editable(obj, request):
        response = _("Permission denied")
    elif form.is_valid():
        form.save()
        response = ""
    else:
        response = form.errors.values()[0][0]
    return HttpResponse(unicode(response))




from django.contrib.admin.views.decorators import LOGIN_FORM_KEY
from django.http import HttpResponseRedirect
from django.template import TemplateDoesNotExist
from django.template.loader import get_template

from mezzanine.settings import MOBILE_USER_AGENTS


class MobileTemplate(object):
    """
    If a mobile user agent is detected, inspect the default args for the view
    func, and if a template name is found assume it is the template arg and
    attempt to load a mobile template based on the original template name.
    """
    def process_view(self, request, view_func, view_args, view_kwargs):
        user_agent = request.META.get("HTTP_USER_AGENT", "")
        if [check for check in MOBILE_USER_AGENTS if check in user_agent]:
            template = view_kwargs.get("template")
            if template is None and view_func.func_defaults is not None:
                for default in view_func.func_defaults:
                    if str(default).endswith(".html"):
                        template = default
            if template is not None:
                template = template.rsplit(".html", 1)[0] + ".mobile.html"
                try:
                    get_template(template)
                except TemplateDoesNotExist:
                    pass
                else:
                    view_kwargs["template"] = template
                    return view_func(request, *view_args, **view_kwargs)
        return None


class AdminLoginInterfaceSelector(object):
    """
    Checks for a POST from the admin login view and if authentication is
    successful and the "site" interface is selected, redirect to the site.
    """
    def process_view(self, request, view_func, view_args, view_kwargs):
        is_login = LOGIN_FORM_KEY in request.POST
        is_logged_in = request.user.is_authenticated()
        site_selected = request.POST.get("interface") == "site"
        if is_login and not is_logged_in and site_selected:
            response = view_func(request, *view_args, **view_kwargs)
            if request.user.is_authenticated():
                return HttpResponseRedirect(request.GET.get("next", "/"))
            else:
                return response
        return None




from django.db import models
from django.utils.translation import ugettext_lazy as _

from mezzanine.core.fields import HtmlField
from mezzanine.core.models import Orderable, Content
from mezzanine.forms.settings import FIELD_MAX_LENGTH, LABEL_MAX_LENGTH
from mezzanine.pages.models import Page


FIELD_CHOICES = (
    ("CharField", _("Single line text")),
    ("CharField/django.forms.Textarea", _("Multi line text")),
    ("EmailField", _("Email")),
    ("BooleanField", _("Check box")),
    ("MultipleChoiceField/django.forms.CheckboxSelectMultiple", 
        _("Check boxes")),
    ("ChoiceField", _("Drop down")),
    ("MultipleChoiceField", _("Multi select")),
    ("ChoiceField/django.forms.RadioSelect", _("Radio buttons")),
    ("FileField", _("File upload")),
    ("DateField/django.forms.extras.SelectDateWidget", _("Date")),
    ("DateTimeField", _("Date/time")),
    ("CharField/django.forms.HiddenInput", _("Hidden")),
)


class Form(Page, Content):
    """
    A user-built form.
    """

    button_text = models.CharField(_("Button text"), max_length=50, 
        default=_("Submit"))
    response = HtmlField(_("Response"))
    send_email = models.BooleanField(_("Send email"), default=True,
        help_text=_("If checked, the person entering the form will be sent an "
                                                                    "email"))
    email_from = models.EmailField(_("From address"), blank=True,
        help_text=_("The address the email will be sent from"))
    email_copies = models.CharField(_("Send copies to"), blank=True,
        help_text=_("One or more email addresses, separated by commas"),
        max_length=200)
    email_subject = models.CharField(_("Subject"), max_length=200, blank=True)
    email_message = models.TextField(_("Message"), blank=True)

    class Meta:
        verbose_name = _("Form")
        verbose_name_plural = _("Forms")


class FieldManager(models.Manager):
    """
    Only show visible fields when displaying actual form..
    """
    def visible(self):
        return self.filter(visible=True)


class Field(Orderable):
    """
    A field for a user-built form.
    """

    form = models.ForeignKey("Form", related_name="fields")
    label = models.CharField(_("Label"), max_length=LABEL_MAX_LENGTH)
    field_type = models.CharField(_("Type"), choices=FIELD_CHOICES,
        max_length=55)
    required = models.BooleanField(_("Required"), default=True)
    visible = models.BooleanField(_("Visible"), default=True)
    choices = models.CharField(_("Choices"), max_length=1000, blank=True,
        help_text="Comma separated options where applicable")
    default = models.CharField(_("Default value"), blank=True, 
        max_length=FIELD_MAX_LENGTH)
    help_text = models.CharField(_("Help text"), blank=True, max_length=100)

    objects = FieldManager()

    class Meta:
        verbose_name = _("Field")
        verbose_name_plural = _("Fields")
        order_with_respect_to = "form"

    def __unicode__(self):
        return self.label


class FormEntry(models.Model):
    """
    An entry submitted via a user-built form.
    """

    form = models.ForeignKey("Form", related_name="entries")
    entry_time = models.DateTimeField(_("Date/time"))

    class Meta:
        verbose_name = _("Form entry")
        verbose_name_plural = _("Form entries")


class FieldEntry(models.Model):
    """
    A single field value for a form entry submitted via a user-built form.
    """

    entry = models.ForeignKey("FormEntry", related_name="fields")
    field_id = models.IntegerField()
    value = models.CharField(max_length=FIELD_MAX_LENGTH)

    class Meta:
        verbose_name = _("Form field entry")
        verbose_name_plural = _("Form field entries")



#!/usr/bin/env python

from __future__ import with_statement
import os
from optparse import OptionParser
from distutils.dir_util import copy_tree
from shutil import move
from uuid import uuid4


def create_project():
    """
    Copies the contents of the project_template directory to a new directory
    specified as an argument to the command line.
    """

    parser = OptionParser(usage="usage: %prog [options] project_name")
    parser.add_option("-a", "--alternate", dest="alt", metavar="PACKAGE", 
        help="Alternate package to use, containing a project_template")
    parser.add_option("-s", "--source", dest="copy_source", default=False,
        action="store_true", help="Copy package source to new project")
    parser.add_option("-t", "--templates", dest="copy_templates", default=True,
        action="store_false", help="Copy templates to the project [default]")
    (options, args) = parser.parse_args()

    if len(args) != 1:
        parser.error("project_name must be specified")
    project_name = args[0]
    if project_name.startswith("-"):
        parser.error("project_name cannot start with '-'")

    # Ensure the given directory name doesn't clash with an existing Python
    # package/module.
    try:
        __import__(project_name)
    except ImportError:
        pass
    else:
        parser.error("'%s' conflicts with the name of an existing "
            "Python module and cannot be used as a project name. Please try "
            "another name." % project_name)
    
    packages = ["mezzanine"]
    if options.alt:
        packages.append(options.alt)
    for package_name in packages:
        try:
            __import__(package_name)
        except ImportError:
            parser.error("Could not import package '%s'" % package_name) 
    for package_name in packages:
        package = __import__(package_name)
        package_path = os.path.dirname(os.path.abspath(package.__file__))
        from_path = os.path.join(package_path, "project_template")
        to_path = os.path.join(os.getcwd(), project_name)
        if options.copy_source:
            copy_tree(package_path, os.path.join(to_path, package_name))
        copy_tree(from_path, to_path)
        move(os.path.join(to_path, "local_settings.py.template"),
            os.path.join(to_path, "local_settings.py"))
        if options.copy_templates:
            to_path = os.path.join(to_path, "templates")
            for app_dir in os.listdir(package_path):
                template_dir = os.path.join(package_path, app_dir, "templates")
                if os.path.isdir(template_dir):
                    copy_tree(template_dir, to_path)
    settings_path = os.path.join(os.getcwd(), project_name, "settings.py")
    with open(settings_path, "r") as f:
        data = f.read()
    with open(settings_path, "w") as f:
        secret_key = "%s%s%s" % (uuid4(), uuid4(), uuid4())
        f.write(data.replace("%(SECRET_KEY)s", secret_key))


if __name__ == "__main__":
    create_project()





from copy import deepcopy

from django.contrib import admin

from mezzanine.blog.models import BlogPost, BlogCategory, Comment
from mezzanine.core.admin import DisplayableAdmin, OwnableAdmin
from mezzanine.settings import COMMENTS_DISQUS_SHORTNAME


blogpost_fieldsets = deepcopy(DisplayableAdmin.fieldsets)
blogpost_fieldsets[0][1]["fields"].insert(1, "category")
blogpost_fieldsets[0][1]["fields"].append("content")
blogpost_radio_fields = deepcopy(DisplayableAdmin.radio_fields)
blogpost_radio_fields["category"] = admin.HORIZONTAL


class BlogPostAdmin(DisplayableAdmin, OwnableAdmin):

    fieldsets = blogpost_fieldsets
    list_display = ("title", "user", "status", "admin_link")
    radio_fields = blogpost_radio_fields

    def save_form(self, request, form, change):
        """
        Super class ordering is important here - user must get saved first.
        """
        OwnableAdmin.save_form(self, request, form, change)
        return DisplayableAdmin.save_form(self, request, form, change)


class BlogCategoryAdmin(admin.ModelAdmin):

    fieldsets = ((None, {"fields": ("title",)}),)

    def in_menu(self):
        """
        Hide from the admin menu.
        """
        return False

class CommentAdmin(admin.ModelAdmin):

    list_display = ("avatar_link", "intro", "time_created", "approved",
        "blog_post", "admin_link")
    list_display_links = ("intro", "time_created")
    list_editable = ("approved",)
    list_filter = ("blog_post", "approved", "name")
    search_fields = ("name", "email", "body")
    date_hierarchy = "time_created"
    ordering = ("-time_created",)
    fieldsets = (
        (None, {"fields": (("name", "email", "website"), "body",
            ("ip_address", "approved"), ("blog_post", "replied_to"))}),
    )

admin.site.register(BlogPost, BlogPostAdmin)
admin.site.register(BlogCategory, BlogCategoryAdmin)
if not COMMENTS_DISQUS_SHORTNAME:
    admin.site.register(Comment, CommentAdmin)




from datetime import datetime

from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.db.models.base import ModelBase
from django.template.defaultfilters import slugify, truncatewords_html
from django.utils.translation import ugettext, ugettext_lazy as _

from mezzanine.core.fields import HtmlField
from mezzanine.core.managers import DisplayableManager, KeywordManager
from mezzanine.settings import CONTENT_STATUS_CHOICES, CONTENT_STATUS_DRAFT
from mezzanine.utils import base_concrete_model


class Slugged(models.Model):
    """
    Abstract model that handles auto-generating slugs.
    """

    title = models.CharField(_("Title"), max_length=100)
    slug = models.CharField(_("URL"), max_length=100, blank=True, null=True)

    class Meta:
        abstract = True
        ordering = ("title",)

    def __unicode__(self):
        return self.title

    def save(self, *args, **kwargs):
        """
        Create a unique slug from the title by appending an index.
        """
        if not self.slug:
            # For custom content types, use the ``Page`` instance for slug
            # lookup.
            concrete_model = base_concrete_model(Slugged, self)
            self.slug = self.get_slug()
            i = 0
            while True:
                if i > 0:
                    if i > 1:
                        self.slug = self.slug.rsplit("-", 1)[0]
                    self.slug = "%s-%s" % (self.slug, i)
                qs = concrete_model.objects.all()
                if self.id is not None:
                    qs = qs.exclude(id=self.id)
                try:
                    qs.get(slug=self.slug)
                except ObjectDoesNotExist:
                    break
                i += 1
        super(Slugged, self).save(*args, **kwargs)

    def natural_key(self):
        return (self.slug,)

    def get_slug(self):
        return slugify(self.title)

class Displayable(Slugged):
    """
    Abstract model that provides features of a visible page on the website
    such as publishing fields and meta data.
    """

    status = models.IntegerField(_("Status"),
        choices=CONTENT_STATUS_CHOICES, default=CONTENT_STATUS_DRAFT)
    publish_date = models.DateTimeField(_("Published from"), 
        help_text=_("With published selected, won't be shown until this time"),
        blank=True, null=True)
    expiry_date = models.DateTimeField(_("Expires on"), 
        help_text=_("With published selected, won't be shown after this time"),
        blank=True, null=True)
    description = HtmlField(_("Description"), blank=True)
    keywords = models.ManyToManyField("Keyword", verbose_name=_("Keywords"),
        blank=True)
    _keywords = models.CharField(max_length=500, editable=False)
    short_url = models.URLField(blank=True, null=True)

    objects = DisplayableManager()
    search_fields = {"_keywords": 10, "title": 5}

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        """
        Create the description from the content if none given.
        """
        if self.publish_date is None:
            # publish_date will be blank when a blog post is created from the
            # quick blog form in the admin dashboard.
            self.publish_date = datetime.now()
        if not self.description:
            for field_type in (HtmlField, models.TextField):
                if not self.description:
                    for field in self._meta.fields:
                        if isinstance(field, field_type):
                            self.description = getattr(self, field.name)
                            if self.description:
                                break
            if not self.description:
                self.description = self.title
            for end in ("</p>", "<br />", "\n", ". "):
                if end in self.description:
                    self.description = self.description.split(end)[0] + end
                    break
            else:
                self.description = truncatewords_html(self.description, 100)
        super(Displayable, self).save(*args, **kwargs)

    def set_searchable_keywords(self):
        """
        Stores the keywords as a single string into the ``_keywords`` field
        for convenient access when searching.
        """
        self._keywords = " ".join([kw.title for kw in self.keywords.all()])
        self.save()

    def admin_link(self):
        return "<a href='%s'>%s</a>" % (self.get_absolute_url(),
            ugettext("View on site"))
    admin_link.allow_tags = True
    admin_link.short_description = ""


class Content(models.Model):
    """
    Provides a HTML field for manging general content and making it searchable.
    """

    content = HtmlField(_("Content"))

    search_fields = ("content",)

    class Meta:
        abstract = True


class OrderableBase(ModelBase):
    """
    Checks for ``order_with_respect_to`` on the model's inner ``Meta`` class
    and if found, copies it to a custom attribute and deletes it since it
    will cause errors when used with ``ForeignKey("self")``. Also creates the
    ``ordering`` attribute on the ``Meta`` class if not yet provided.
    """

    def __new__(cls, name, bases, attrs):
        if "Meta" not in attrs:

            class Meta:
                pass
            attrs["Meta"] = Meta
        if hasattr(attrs["Meta"], "order_with_respect_to"):
            attrs["order_with_respect_to"] = attrs["Meta"].order_with_respect_to
            del attrs["Meta"].order_with_respect_to
        if not hasattr(attrs["Meta"], "ordering"):
            setattr(attrs["Meta"], "ordering", ("_order",))
        return super(OrderableBase, cls).__new__(cls, name, bases, attrs)


class Orderable(models.Model):
    """
    Abstract model that provides a custom ordering integer field similar to
    using Meta's ``order_with_respect_to``, since to date (Django 1.2) this
    doesn't work with ``ForeignKey("self")``. We may also want this feature
    for models that aren't ordered with respect to a particular field.
    """

    __metaclass__ = OrderableBase

    _order = models.IntegerField(_("Order"), null=True)

    class Meta:
        abstract = True

    def with_respect_to(self):
        """
        Returns a dict to use as a filter for ordering operations containing
        the original ``Meta.order_with_respect_to`` value if provided.
        """
        try:
            field = self.order_with_respect_to
            return {field: getattr(self, field)}
        except AttributeError:
            return {}

    def save(self, *args, **kwargs):
        """
        Set the initial ordering value.
        """
        if self._order is None:
            lookup = self.with_respect_to()
            concrete_model = base_concrete_model(Orderable, self)
            self._order = concrete_model.objects.filter(**lookup).count()
        super(Orderable, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        """
        Update the ordering values for siblings.
        """
        lookup = self.with_respect_to()
        lookup["_order__gte"] = self._order
        concrete_model = base_concrete_model(Orderable, self)
        after = concrete_model.objects.filter(**lookup)
        after.update(_order=models.F("_order") - 1)
        super(Orderable, self).delete(*args, **kwargs)


class Ownable(models.Model):
    """
    Abstract model that provides ownership of an object for a user.
    """

    user = models.ForeignKey("auth.User", verbose_name=_("Author"),
        related_name="%(class)ss")

    class Meta:
        abstract = True

    def is_editable(self, request):
        """
        Restrict in-line editing to the objects's owner and superusers.
        """
        return request.user.is_superuser or request.user.id == self.user_id


class Keyword(Slugged):
    """
    Keywords/tags which are managed via a custom Javascript based widget in the
    admin.
    """

    objects = KeywordManager()

    class Meta:
        verbose_name = "Keyword"
        verbose_name_plural = "Keywords"





from urllib import urlopen, urlencode
from uuid import uuid4

from django.conf import settings
from django.contrib import admin
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.core.urlresolvers import reverse, NoReverseMatch
from django.db.models import Model
from django.template import Context
from django.template.loader import get_template
from django.utils.html import strip_tags
from django.utils.simplejson import loads
from django.utils.text import capfirst

from mezzanine import settings as mezzanine_settings
from mezzanine import template
from mezzanine.core.forms import get_edit_form
from mezzanine.utils import decode_html_entities, is_editable

from django.utils.datastructures import SortedDict

register = template.Library()


@register.simple_tag
def setting(setting_name, called_internally=False):
    """
    Return a setting.
    """
    if not called_internally:
        import warnings
        warnings.warn(
            "The ``setting`` tag will be deprecated - " \
            "please use the ``load_settings`` tag.", DeprecationWarning)
    value = getattr(mezzanine_settings, setting_name,
        getattr(settings, setting_name, None))
    if value is None:
        value = ""
    return value


@register.render_tag
def load_settings(context, token):
    """
    Push the given setting names into the context.
    """
    for setting_name in token.split_contents()[1:]:
        context[setting_name] = setting(setting_name, called_internally=True)
    return ""


@register.render_tag
def set_short_url_for(context, token):
    """
    Sets the ``short_url`` attribute of the given model using the bit.ly
    credentials if they have been specified and saves it.
    """
    obj = context[token.split_contents()[1]]
    request = context["request"]
    if getattr(obj, "short_url") is None:
        obj.short_url = request.build_absolute_uri(request.path)
        args = {
            "login": getattr(mezzanine_settings, "BLOG_BITLY_USER"),
            "apiKey": getattr(mezzanine_settings, "BLOG_BITLY_KEY"),
            "longUrl": obj.short_url,
        }
        if args["login"] and args["apiKey"]:
            url = "http://api.bit.ly/v3/shorten?%s" % urlencode(args)
            response = loads(urlopen(url).read())
            if response["status_code"] == 200:
                obj.short_url = response["data"]["url"]
                obj.save()
    return ""


@register.to_end_tag
def metablock(parsed):
    """
    Remove HTML tags, entities and superfluous characters from meta blocks.
    """
    parsed = " ".join(parsed.replace("\n", "").split()).replace(" ,", ",")
    return strip_tags(decode_html_entities(parsed))


@register.inclusion_tag("includes/pagination.html", takes_context=True)
def pagination_for(context, current_page):
    """
    Include the pagination template and data for persisting querystring in
    pagination links.
    """
    querystring = context["request"].GET.copy()
    if "page" in querystring:
        del querystring["page"]
    querystring = querystring.urlencode()
    return {"current_page": current_page, "querystring": querystring}


@register.inclusion_tag("includes/editable_loader.html", takes_context=True)
def editable_loader(context):
    """
    Set up the required JS/CSS for the in-line editing toolbar and controls.
    """
    t = get_template("includes/editable_toolbar.html")
    context["REDIRECT_FIELD_NAME"] = REDIRECT_FIELD_NAME
    context["toolbar"] = t.render(Context(context))
    return context


@register.to_end_tag
def editable(parsed, context, token):
    """
    Add the required HTML to the parsed content for in-line editing, such as
    the icon and edit form if the object is deemed to be editable - either it
    has an ``editable`` method which returns True, or the logged in user has
    change permissions for the model.
    """
    parts = token.split_contents()[1].split(".")
    obj = context[parts.pop(0)]
    attr = parts.pop()
    while parts:
        obj = getattr(obj, parts.pop(0))
    if not parsed.strip():
        parsed = getattr(obj, attr)
    if isinstance(obj, Model) and is_editable(obj, context["request"]):
        context["form"] = get_edit_form(obj, attr)
        context["original"] = parsed
        context["uuid"] = uuid4()
        t = get_template("includes/editable_form.html")
        return t.render(Context(context))
    return parsed


@register.inclusion_tag("admin/includes/dropdown_menu.html", takes_context=True)
def admin_dropdown_menu(context):
    """
    Adopted from ``django.contrib.admin.sites.AdminSite.index``. Renders a 
    list of lists of models grouped and ordered according to 
    ``mezzanine.settings.ADMIN_MENU_ORDER``.
    """
    app_dict = {}
    user = context["request"].user
    for model, model_admin in admin.site._registry.items():
        app_label = model._meta.app_label
        in_menu = not hasattr(model_admin, "in_menu") or model_admin.in_menu()
        if in_menu and user.has_module_perms(app_label):
            perms = model_admin.get_model_perms(context["request"])
            admin_url = ""
            if perms["change"]:
                admin_url = "changelist"
            elif perms["add"]:
                admin_url = "add"
            if admin_url:
                model_label = "%s.%s" % (app_label, model.__name__)
                for (name, items) in mezzanine_settings.ADMIN_MENU_ORDER:
                    try:
                        index = items.index(model_label)
                    except ValueError:
                        pass
                    else:
                        app_title = name
                        break
                else:
                    index = None
                    app_title = app_label
                model_dict = {
                    "index": index,
                    "name": capfirst(model._meta.verbose_name_plural),
                    "admin_url": reverse("admin:%s_%s_%s" % (
                        app_label, model.__name__.lower(), admin_url))
                }
                app_title = app_title.title()
                if app_title in app_dict:
                    app_dict[app_title]["models"].append(model_dict)
                else:
                    try:
                        titles = [x[0] for x in 
                            mezzanine_settings.ADMIN_MENU_ORDER]
                        index = titles.index(app_title)
                    except ValueError:
                        index = None
                    app_dict[app_title] = {
                        "index": index,
                        "name": app_title,
                        "models": [model_dict],
                    }
    app_list = app_dict.values()
    sort = lambda x: x["name"] if x["index"] is None else x["index"]
    for app in app_list:
        app["models"].sort(key=sort)
    app_list.sort(key=sort)
    # This silliness is required for the test suite to pass. For some reason 
    # the admin URLs can't be resolved while testing so in this situation,
    # trapping it and using an empty string for the index URL is quite 
    # acceptable.
    try:
        admin_url = reverse("admin:index")
    except NoReverseMatch:
        admin_url = ""
    return {"app_list": app_list, "admin_url": admin_url}




import os.path

from django.utils.translation import ugettext_lazy as _
from django.conf import settings


def setting(setting_name, default_value):
    """
    Set the default value for a setting, using the value found in the project
    settings module prefixed with MEZZANINE_ if found.
    """
    globals()[setting_name] = getattr(settings,
        "MEZZANINE_%s" % setting_name, default_value)

# Unregister these models installed by default (occurs in urlconf).
setting("ADMIN_REMOVAL", ())

# Controls the ordering and grouping of the admin menu.
setting("ADMIN_MENU_ORDER", (
    (_("Content"), ("pages.Page", "blog.BlogPost", "blog.Comment",)),
    (_("Site"), ("auth.User", "auth.Group", "sites.Site", 
        "redirects.Redirect")),
))

# Credentials for bit.ly URL shortening service.
setting("BLOG_BITLY_USER", None)
setting("BLOG_BITLY_KEY", None)

# Number of blog posts to show on a blog listing page.
setting("BLOG_POST_PER_PAGE", 5)

# Maximum number of paging links to show on a blog listing page.
setting("BLOG_POST_MAX_PAGING_LINKS", 10)

# Slug of the page object for the blog.
setting("BLOG_SLUG", "blog")

# Shortname when using the Disqus comments system (http://disqus.com).
setting("COMMENTS_DISQUS_SHORTNAME", None)

# Disqus user's API key for displaying recent comments in the admin dashboard.
setting("COMMENTS_DISQUS_KEY", None)

# If True, the built-in comments are approved by default.
setting("COMMENTS_DEFAULT_APPROVED", True)

# Number of latest comments to show in the admin dashboard.
setting("COMMENTS_NUM_LATEST", 5)

# If True, unapproved comments will have a placeholder visible on the site
# with a "waiting for approval" or "comment removed" message based on the
# workflow around the ``MEZZANINE_COMMENTS_DEFAULT_APPROVED`` setting - if
# True then the former message is used, if False then the latter.
setting("COMMENTS_UNAPPROVED_VISIBLE", True)

# Media files for admin.
CONTENT_MEDIA_PATH = os.path.join(os.path.dirname(__file__), "core", "media")

# URL for serving internal media files.
setting("CONTENT_MEDIA_URL", "/content_media/")

# Content status choices.
CONTENT_STATUS_DRAFT = 1
CONTENT_STATUS_PUBLISHED = 2
CONTENT_STATUS_CHOICES = (
    (CONTENT_STATUS_DRAFT, _("Draft")),
    (CONTENT_STATUS_PUBLISHED, _("Published")),
)

# ID for using Google Analytics (http://www.google.com/analytics/) referred to
# as "Web Property ID"
setting("GOOGLE_ANALYTICS_ID", None)

# Strings to check for in user agents when testing for a mobile device.
setting("MOBILE_USER_AGENTS", ("2.0 MMP", "240x320", "400X240", "AvantGo",
    "BlackBerry", "Blazer", "Cellphone", "Danger", "DoCoMo", "Elaine/3.0",
    "EudoraWeb", "Googlebot-Mobile", "hiptop", "IEMobile", "KYOCERA/WX310K",
    "LG/U990", "MIDP-2.", "MMEF20", "MOT-V", "NetFront", "Newt", "Nintendo Wii",
    "Nitro", "Nokia", "Opera Mini", "Palm", "PlayStation Portable", "portalmmm",
    "Proxinet", "ProxiNet", "SHARP-TQ-GX10", "SHG-i900", "Small",
    "SonyEricsson", "Symbian OS", "SymbianOS", "TS21i-10", "UP.Browser",
    "UP.Link", "webOS", "Windows CE", "WinWAP", "YahooSeeker/M1A1-R2D2",
    "iPhone", "iPod", "Android", "BlackBerry9530", "LG-TU915 Obigo", "LGE VX",
    "webOS", "Nokia5800"))

# Number of different sizes given to tags when shown as a cloud.
setting("TAG_CLOUD_SIZES", 4)

# If True, the pages menu will show all levels of navigation by default,
# otherwise child pages are only shown when viewing the parent page.
setting("PAGES_MENU_SHOW_ALL", True)

# Number of results to show in the search results page.
setting("SEARCH_PER_PAGE", 10)

# Maximum number of paging links to show in the search results page.
setting("SEARCH_MAX_PAGING_LINKS", 10)

# List of words which will be stripped from search queries.
setting("STOP_WORDS", ("a", "about", "above", "above", "across", "after",
    "afterwards", "again", "against", "all", "almost", "alone", "along",
    "already", "also", "although", "always", "am", "among", "amongst",
    "amoungst", "amount", "an", "and", "another", "any", "anyhow", "anyone",
    "anything", "anyway", "anywhere", "are", "around", "as", "at", "back",
    "be", "became", "because", "become", "becomes", "becoming", "been",
    "before", "beforehand", "behind", "being", "below", "beside", "besides",
    "between", "beyond", "bill", "both", "bottom", "but", "by", "call", "can",
    "cannot", "cant", "co", "con", "could", "couldnt", "cry", "de", "describe",
    "detail", "do", "done", "down", "due", "during", "each", "eg", "eight",
    "either", "eleven", "else", "elsewhere", "empty", "enough", "etc", "even",
    "ever", "every", "everyone", "everything", "everywhere", "except", "few",
    "fifteen", "fify", "fill", "find", "fire", "first", "five", "for", "former",
    "formerly", "forty", "found", "four", "from", "front", "full", "further",
    "get", "give", "go", "had", "has", "hasnt", "have", "he", "hence", "her",
    "here", "hereafter", "hereby", "herein", "hereupon", "hers", "herself",
    "him", "himself", "his", "how", "however", "hundred", "ie", "if", "in",
    "inc", "indeed", "interest", "into", "is", "it", "its", "itself", "keep",
    "last", "latter", "latterly", "least", "less", "ltd", "made", "many", "may",
    "me", "meanwhile", "might", "mill", "mine", "more", "moreover", "most",
    "mostly", "move", "much", "must", "my", "myself", "name", "namely",
    "neither", "never", "nevertheless", "next", "nine", "no", "nobody", "none",
    "noone", "nor", "not", "nothing", "now", "nowhere", "of", "off", "often",
    "on", "once", "one", "only", "onto", "or", "other", "others", "otherwise",
    "our", "ours", "ourselves", "out", "over", "own", "part", "per", "perhaps",
    "please", "put", "rather", "re", "same", "see", "seem", "seemed", "seeming",
    "seems", "serious", "several", "she", "should", "show", "side", "since",
    "sincere", "six", "sixty", "so", "some", "somehow", "someone", "something",
    "sometime", "sometimes", "somewhere", "still", "such", "system", "take",
    "ten", "than", "that", "the", "their", "them", "themselves", "then",
    "thence", "there", "thereafter", "thereby", "therefore", "therein",
    "thereupon", "these", "they", "thickv", "thin", "third", "this", "those",
    "though", "three", "through", "throughout", "thru", "thus", "to",
    "together", "too", "top", "toward", "towards", "twelve", "twenty", "two",
    "un", "under", "until", "up", "upon", "us", "very", "via", "was", "we",
    "well", "were", "what", "whatever", "when", "whence", "whenever", "where",
    "whereafter", "whereas", "whereby", "wherein", "whereupon", "wherever",
    "whether", "which", "while", "whither", "who", "whoever", "whole", "whom",
    "whose", "why", "will", "with", "within", "without", "would", "yet", "you",
    "your", "yours", "yourself", "yourselves", "the"))




from django.db.models import TextField


class HtmlField(TextField):
    """
    TextField that stores HTML.
    """

    def formfield(self, **kwargs):
        """
        Apply the class to the widget that will render the field as a
        TincyMCE Editor.
        """
        formfield = super(HtmlField, self).formfield(**kwargs)
        formfield.widget.attrs["class"] = "mceEditor"
        return formfield




from collections import defaultdict

from django.core.urlresolvers import reverse
from django.db.models import get_model, get_models

from mezzanine import template
from mezzanine.pages.models import Page
from mezzanine.settings import PAGES_MENU_SHOW_ALL


register = template.Library()


def _page_menu(context, parent_page):
    """
    Return a list of child pages for the given parent, storing all
    pages in a dict in the context when first called using parents as keys
    for retrieval on subsequent recursive calls from the menu template.
    """
    if "menu_pages" not in context:
        pages = defaultdict(list)
        try:
            user = context["request"].user
        except KeyError:
            user = None
        try:
            slug = context["request"].path.strip("/")
        except KeyError:
            slug = ""
        for page in Page.objects.published(for_user=user).order_by("_order"):
            setattr(page, "selected", (slug + "/").startswith(page.slug + "/"))
            setattr(page, "primary", page.parent_id is None)
            pages[page.parent_id].append(page)
        context["menu_pages"] = pages
    if parent_page is not None:
        parent_page = parent_page.id
    context["page_branch"] = context["menu_pages"].get(parent_page, [])
    context["PAGES_MENU_SHOW_ALL"] = PAGES_MENU_SHOW_ALL
    return context


@register.inclusion_tag("pages/includes/tree_menu.html", takes_context=True)
def tree_menu(context, parent_page=None):
    """
    Tree menu that renders all pages in the navigation hierarchically.
    """
    return _page_menu(context, parent_page)


@register.inclusion_tag("pages/includes/primary_menu.html", takes_context=True)
def primary_menu(context, parent_page=None):
    """
    Page menu that only renders the primary top-level pages.
    """
    return _page_menu(context, parent_page)


@register.inclusion_tag("pages/includes/footer_menu.html", takes_context=True)
def footer_menu(context, parent_page=None):
    """
    Page menu that only renders the footer pages.
    """
    return _page_menu(context, parent_page)


@register.inclusion_tag("pages/includes/breadcrumb_menu.html", 
    takes_context=True)
def breadcrumb_menu(context, parent_page=None):
    """
    Page menu that only renders the pages that are parents of the current 
    page, as well as the current page itself.
    """
    return _page_menu(context, parent_page)


@register.inclusion_tag("admin/includes/tree_menu.html", takes_context=True)
def tree_menu_admin(context, parent_page=None):
    """
    Admin tree menu for managing pages.
    """
    return _page_menu(context, parent_page)


@register.as_tag
def models_for_pages(*args):
    """
    Create a select list containing each of the models that subclass the
    ``Page`` model.
    """
    page_models = []
    for model in get_models():
        if model is not Page and issubclass(model, Page):
            setattr(model, "name", model._meta.verbose_name)
            setattr(model, "add_url", reverse("admin:%s_%s_add" %
                (model._meta.app_label, model.__name__.lower())))
            page_models.append(model)
    return page_models


@register.filter
def is_page_content_model(admin_model_dict):
    """
    Returns True if the model in the given admin dict is a subclass of the
    ``Page`` model.
    """
    args = admin_model_dict["admin_url"].strip("/").split("/")
    if len(args) == 2:
        model = get_model(*args)
        return model is not Page and issubclass(model, Page)
    return False




from datetime import datetime
from urllib import urlopen
from urlparse import urlparse

from django.contrib.auth.models import User
from django.contrib.redirects.models import Redirect
from django.contrib.sites.models import Site
from django.core.management.base import BaseCommand, CommandError
from django.utils.simplejson import loads


class Command(BaseCommand):
    """
    Import Tumblr blog posts into the blog app.
    """

    help = "Import Tumblr blog posts into the blog app."
    args = "tumblr_user mezzanine_user"

    def handle(self, tumblr_user="", mezzanine_user="", *args, **options):

        json_url = "http://%s.tumblr.com/api/read/json" % tumblr_user
        if not (tumblr_user and mezzanine_user):
            raise CommandError("Usage is import_tumblr %s" % self.args)
        try:
            response = urlopen(json_url)
            if response.code != 200:
                raise IOError
        except IOError:
            raise CommandError("Invalid Tumblr user")
        if mezzanine_user is not None:
            try:
                mezzanine_user = User.objects.get(username=mezzanine_user)
            except User.DoesNotExist:
                raise CommandError("Invalid Mezzanine user")

        from mezzanine.blog.models import BlogPost
        from mezzanine.core.models import Keyword
        from mezzanine.settings import CONTENT_STATUS_PUBLISHED
        site = Site.objects.get_current()
        start = "var tumblr_api_read ="
        date_format = "%a, %d %b %Y %H:%M:%S"
        json = loads(response.read().split(start, 1)[1].strip().rstrip(";"))
        for entry in json["posts"]:
            if entry["type"] == "regular":
                print "Importing %s" % entry["regular-title"]
                post, created = BlogPost.objects.get_or_create(
                    user=mezzanine_user, title=entry["regular-title"],
                    content=entry["regular-body"],
                    status=CONTENT_STATUS_PUBLISHED,
                    publish_date=datetime.strptime(entry["date"], date_format))
                for tag in entry["tags"]:
                    keyword, created = Keyword.objects.get_or_create(title=tag)
                    post.keywords.add(keyword)
                post.set_searchable_keywords()
                redirect, created = Redirect.objects.get_or_create(site=site,
                    old_path=urlparse(entry["url-with-slug"]).path)
                redirect.new_path = urlparse(post.get_absolute_url()).path
                redirect.save()




from datetime import datetime
from os.path import join
from uuid import uuid4

from django import forms
from django.core.files.storage import FileSystemStorage
from django.utils.importlib import import_module

from mezzanine.forms.models import FormEntry
from mezzanine.forms.settings import FIELD_MAX_LENGTH, UPLOAD_ROOT


fs = FileSystemStorage(location=UPLOAD_ROOT)


class FormForForm(forms.ModelForm):

    class Meta:
        model = FormEntry
        exclude = ("form", "entry_time")

    def __init__(self, form, *args, **kwargs):
        """
        Dynamically add each of the form fields for the given form model
        instance and its related field model instances.
        """
        self.form = form
        self.form_fields = form.fields.visible()
        super(FormForForm, self).__init__(*args, **kwargs)
        for field in self.form_fields:
            field_key = "field_%s" % field.id
            if "/" in field.field_type:
                field_class_name, field_widget = field.field_type.split("/")
            else:
                field_class_name, field_widget = field.field_type, None
            field_class = getattr(forms, field_class_name)
            field_args = {"label": field.label, "required": field.required,
                "help_text": field.help_text}
            arg_names = field_class.__init__.im_func.func_code.co_varnames
            if "max_length" in arg_names:
                field_args["max_length"] = FIELD_MAX_LENGTH
            if "choices" in arg_names:
                choices = field.choices.split(",")
                field_args["choices"] = zip(choices, choices)
            if field_widget is not None:
                module, widget = field_widget.rsplit(".", 1)
                field_args["widget"] = getattr(import_module(module), widget)
            self.initial[field_key] = field.default
            self.fields[field_key] = field_class(**field_args)
            # Add identifying CSS classes to the field.
            css_class = field_class_name.lower()
            if field.required:
                css_class += " required"
            self.fields[field_key].widget.attrs["class"] = css_class

    def save(self, **kwargs):
        """
        Create a FormEntry instance and related FieldEntry instances for each
        form field.
        """
        entry = super(FormForForm, self).save(commit=False)
        entry.form = self.form
        entry.entry_time = datetime.now()
        entry.save()
        for field in self.form_fields:
            field_key = "field_%s" % field.id
            value = self.cleaned_data[field_key]
            if value and self.fields[field_key].widget.needs_multipart_form:
                value = fs.save(join("forms", str(uuid4()), value.name), value)
            if isinstance(value, list):
                value = ", ".join([v.strip() for v in value])
            if value:
                entry.fields.create(field_id=field.id, value=value)
        return entry

    def email_to(self):
        """
        Return the value entered for the first field of type EmailField.
        """
        for field in self.form_fields:
            field_class = field.field_type.split("/")[0]
            if field_class == "EmailField":
                return self.cleaned_data["field_%s" % field.id]
        return None




from django.core.urlresolvers import resolve
from django.db import models
from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _

from mezzanine.core.models import Displayable, Orderable, Content


class Page(Orderable, Displayable):
    """
    A page in the page tree.
    """

    parent = models.ForeignKey("Page", blank=True, null=True,
        related_name="children")
    in_navigation = models.BooleanField(_("Show in navigation"), default=True)
    in_footer = models.BooleanField(_("Show in footer"))
    titles = models.CharField(editable=False, max_length=1000, null=True)
    content_model = models.CharField(editable=False, max_length=50, null=True)
    login_required = models.BooleanField(_("Login required"), 
        help_text=_("If checked, only logged in users can view this page"))

    class Meta:
        verbose_name = _("Page")
        verbose_name_plural = _("Pages")
        ordering = ("titles",)
        order_with_respect_to = "parent"

    def __unicode__(self):
        return self.titles

    @models.permalink
    def get_absolute_url(self):
        return ("page", (), {"slug": self.get_slug()})

    def save(self, *args, **kwargs):
        """
        Create the titles field using the titles up the parent chain and set
        the initial value for ordering.
        """
        if self.id is None:
            self.content_model = self._meta.object_name.lower()
        titles = [self.title]
        parent = self.parent
        while parent is not None:
            titles.insert(0, parent.title)
            parent = parent.parent
        self.titles = " / ".join(titles)
        super(Page, self).save(*args, **kwargs)

    def get_content_model(self):
        return getattr(self, self.content_model, None)

    def get_slug(self):
        """
        Recursively build the slug from the chain of parents.
        """
        slug = slugify(self.title)
        if self.parent is not None:
            return "%s/%s" % (self.parent.get_slug(), slug)
        return slug

    def overridden(self):
        """
        Return True if the page's slug has an explicitly defined url pattern
        and is therefore considered to be overriden.
        """
        from mezzanine.pages.views import page
        resolved_view = resolve(self.get_absolute_url())[0]
        return resolved_view != page
        

class ContentPage(Page, Content):
    """
    Implements the default type of page with a single HTML content field.
    """

    class Meta:
        verbose_name = _("Content")
        verbose_name_plural = _("Content pages")

